/**************************************************************************************
Author:			David Szybala
Program:		Geometry_Linker.ms
Date:			8/14/13
Description:	This script will assist in parenting or creating instances.
***************************************************************************************/

try (destroyDialog geometry_linker_rollout) catch()


(--put script into local scope
	global geometry_linker_rollout
	local master_objects = #()
	local geometry_linker_ini = getDir #plugcfg + "\\GeometryLinker.ini"
	local geometry_linker_position
	
	rollout geometry_linker_rollout "Geometry Linker"
	(
		groupBox master_group "Masters" pos:[5,5] width:290 height:235
			listBox master_listbox "" pos:[15,25] width:270 height:10
			button add_selected_button "Add Selected Object(s) to List" pos:[15,165] width:270 height:20
			button remove_selected_button "Remove Selected List Item" pos:[15,190] width:270 height:20
			button clear_list_button "Remove All List Items" pos:[15,215] width:270 height:20
		
		groupBox parent_group "Parent" pos:[5,245] width:290 height:45
			button parent_selection_button "Parent Selection to Master" pos:[15,265] width:270 height:20
		
        groupBox instance_group "Instance" pos:[5,295] width:290 height:45
			button instance_selection_button "Convert Selection to Instances of Master" pos:[15,315] width:270 height:20
        
        label status_label "INFO: Standing by..." pos:[5,348] width:290 height:15
            
        /**********************************************************************
        UI move event handler. Stores the current position of the UI
        **********************************************************************/
		on geometry_linker_rollout moved pos do
            setIniSetting geometry_linker_ini "Dialog" "Position" (pos as string)
		
        /**********************************************************************
        Add master event handler. Stores the current selection as masters
        **********************************************************************/
		on add_selected_button pressed do
		(
			local current_selection = selection as array
			local list_items = master_listbox.items
			
			if current_selection.count > 0 then
			(
				for current in current_selection do
				(
					if (findItem master_objects current) == 0 do
					(
						append master_objects current
						append list_items current.name
					)--end if
				)--end for
				
				master_listbox.items = list_items
                
                status_label.text = "INFO: Selection added to master list"
			)--end if
            else
                status_label.text = "ERROR: No objects selected"
		)--end on
		
        /**********************************************************************
        Remove master event handler. Removes the selected master from the list
        **********************************************************************/
		on remove_selected_button pressed do
		(
			local selected_index = master_listbox.selection
			local list_items = master_listbox.items
			
			if selected_index > 0 then
			(
				deleteItem list_items selected_index
				deleteItem master_objects selected_index
				
				master_listbox.items = list_items
                
                status_label.text = "INFO: Item removed from list"
			)--end if
            else
                status_label.text = "ERROR: No master item selected"
		)--end on
		
        /**********************************************************************
        Clear master event handler. Removes all the master objects from the list
        **********************************************************************/
		on clear_list_button pressed do
		(
			master_objects = #()
			master_listbox.items = #()
			
			status_label.text = "INFO: Master list cleared"
		)--end on
        
        /**********************************************************************
        Parent button event handler. Parents the selected objects to a master
        object
        **********************************************************************/
        on parent_selection_button pressed do
		(
            local selected_index = master_listbox.selection
            local error_counter = 0
            
            if selected_index < 1 do
            (
                status_label.text = "ERROR: No master item selected"
                return false
            ) --end if
            
            if selection.count < 1 do
            (
                status_label.text = "ERROR: No objects selected"
                return false
            ) --end if
            
            for current in selection do
            (
                if current != master_objects[selected_index] do
                (
                    --CRITICAL! Make sure master is not a child of current!
                    if (findItem current.children master_objects[selected_index]) == 0 then
                        current.parent = master_objects[selected_index]
                    else
                    (
                        print("ERROR: Link failure. Master object is a child of " + current.name)
                        error_counter += 1
                    ) --end else
                ) --end if
            ) --end for
                
            if error_counter == 0 then
                status_label.text = "INFO: Selection parented to master"
            else
                status_label.text = ("WARNING: " + (error_counter as string) + " error(s) occurred. Check listener")
		)--end on
		
        /**********************************************************************
        Instance button event handler. Instances the selected objects to a
        master object
        **********************************************************************/
		on instance_selection_button pressed do
		(
            local selected_index = master_listbox.selection
            
            if selected_index < 1 do
            (
                status_label.text = "ERROR: No master item selected"
                return false
            ) --end if
            
            if selection.count < 1 do
            (
                status_label.text = "ERROR: No objects selected"
                return false
            ) --end if
            
            instanceReplace selection master_objects[selected_index]
                
            status_label.text = "INFO: Selection instanced to master"
		)--end on
	)--end rollout
    
	geometry_linker_position = execute (getIniSetting geometry_linker_ini "Dialog" "Position")
	if geometry_linker_position == OK do geometry_linker_position = [100,100]
	if geometry_linker_position.x > sysinfo.desktopsize.x - 300 do
        geometry_linker_position.x = sysinfo.desktopsize.x - 305
	if geometry_linker_position.y > sysinfo.desktopsize.y - 425 do
        geometry_linker_position.y = sysinfo.desktopsize.y - 425
	if geometry_linker_position.x < 0 do geometry_linker_position.x = 0 
	if geometry_linker_position.y < 0 do geometry_linker_position.y = 0
	createDialog geometry_linker_rollout 300 365 geometry_linker_position.x geometry_linker_position.y
)--end local scope
